#pragma once
/**
 * @authors Artur Wasmut, Tobias Frisch, Alexander Gauggel, Vanessa Karolek
 * @file vkcv/DescriptorWrites.hpp
 * @brief Structures to handle descriptor writes.
 */

#include <vector>

#include "Handles.hpp"

namespace vkcv {
	
	struct SampledImageDescriptorWrite {
		inline SampledImageDescriptorWrite(uint32_t binding, ImageHandle image, uint32_t mipLevel = 0, bool useGeneralLayout = false, uint32_t arrayIndex = 0)
		: binding(binding), image(image), mipLevel(mipLevel), useGeneralLayout(useGeneralLayout), arrayIndex(arrayIndex) {};
		uint32_t	binding;
		ImageHandle	image;
		uint32_t    mipLevel;
		bool        useGeneralLayout;
		uint32_t    arrayIndex;
	};

	struct StorageImageDescriptorWrite {
		inline StorageImageDescriptorWrite(uint32_t binding, ImageHandle image, uint32_t mipLevel = 0) 
			: binding(binding), image(image), mipLevel(mipLevel) {};
		uint32_t	binding;
		ImageHandle	image;
		uint32_t	mipLevel;
	};

	struct BufferDescriptorWrite {
		inline BufferDescriptorWrite(uint32_t binding, BufferHandle buffer, bool dynamic = false,
									 uint32_t offset = 0, uint32_t size = 0) :
		binding(binding), buffer(buffer), dynamic(dynamic), offset(offset), size(size) {};
		uint32_t		binding;
		BufferHandle	buffer;
		bool 			dynamic;
		uint32_t 		offset;
		uint32_t 		size;
	};

	struct SamplerDescriptorWrite {
		inline SamplerDescriptorWrite(uint32_t binding, SamplerHandle sampler) : binding(binding), sampler(sampler) {};
		uint32_t		binding;
		SamplerHandle	sampler;
	};

	/**
	 * @brief Only used for RTX. Used to bind the Acceleration Structure.
	 */
	struct AccelerationDescriptorWrite {
	    inline AccelerationDescriptorWrite(uint32_t binding) : binding(binding) {};
	    uint32_t    binding;
	};

	struct DescriptorWrites {
		std::vector<SampledImageDescriptorWrite>		sampledImageWrites;
		std::vector<StorageImageDescriptorWrite>		storageImageWrites;
		std::vector<BufferDescriptorWrite>	    		uniformBufferWrites;
		std::vector<BufferDescriptorWrite>	    		storageBufferWrites;
		std::vector<SamplerDescriptorWrite>			    samplerWrites;
		std::vector<AccelerationDescriptorWrite>        accelerationWrites;
	};
	
}